# -*- mode: makefile; tab-width: 8; -*-

TARGET		= quasar
TEMPLATE	= app

#CONFIG		= qt warn_on
CONFIG		= qt warn_on debug

# for native compilation with ccache:
TMAKE_CC	= ccache gcc
TMAKE_CXX	= ccache g++
TMAKE_LINK	= ccache g++

# for native compilation with pdaxii13's zgcc:
#TMAKE_CC	= gcc
#TMAKE_CXX	= g++
#TMAKE_LINK	= g++

# for pdaXii13 cross compiler:
#TMAKE_CC	= ccache armv5tel-cacko-linux-gcc
#TMAKE_CXX	= ccache armv5tel-cacko-linux-g++
#TMAKE_LINK	= ccache armv5tel-cacko-linux-g++

SQLITE_FLAGS = -DSQLITE_OMIT_UTF16 -DSQLITE_OMIT_AUTHORIZATION -DSQLITE_OMIT_PROGRESS_CALLBACK -DSQLITE_OMIT_SHARED_CACHE -DSQLITE_OMIT_LOAD_EXTENSION -DSQLITE_DEFAULT_TEMP_CACHE_SIZE=1000 -DTHREADSAFE=0 -DSQLITE_ENABLE_MEMORY_MANAGEMENT -O3

TMAKE_CFLAGS	= $$TMAKE_CFLAGS $$SQLITE_FLAGS
TMAKE_CXXFLAGS  = $$TMAKE_CXXFLAGS $$SQLITE_FLAGS

LIBS		= -liconv -lz -lstdc++
INCLUDEPATH	= . compat config tkmix tag/id3lib/include/id3 tag/id3lib/include tag/id3lib sqlite_source
HEADERS		= \
		configuration.h \
		configurationdialog.h \
		media.h \
		mediaidentifier.h \
		mediadatabase.h \
		mediainfodialog.h \
		mplayer.h \
		quasar.h \
		skinpanel.h \
		skinnedbutton.h \
		skinnedlabel.h \
		skinnedtimelabel.h \
		skin.h \
		skinmanager.h \
		operationpanel.h \
		toolpanel.h \
		coverartprovider.h \
		coverart.h \
		playinfo.h \
		imagefx.h \
		videoplaybackwindow.h \
		playlist.h \
		playlistmanager.h \
		playlistoverview.h \
		playlisttoolpanel.h \
		seekbar.h \
		signalhandler.h \
		aboutdialog.h \
		keynames.h \
		action.h \
		textencoding.h \
		textviewdialog.h \
		compat/config.h \
		compat/resource.h \
		config/generalconfig.h \
		config/mplayerconfig.h \
		config/formatsconfig.h \
		config/hotkeyconfig.h \
		config/identifykeydialog.h \
		config/uiconfig.h \
		config/coverartconfig.h \
		tkmix/qtkfiledialog.h \
		system_volume.h \
		tag/id3lib/id3lib_config.h                   \
		tag/id3lib/src/field_def.h                   \
		tag/id3lib/src/field_impl.h                  \
		tag/id3lib/src/flags.h                       \
		tag/id3lib/src/frame_def.h                   \
		tag/id3lib/src/frame_impl.h                  \
		tag/id3lib/src/header.h                      \
		tag/id3lib/src/header_frame.h                \
		tag/id3lib/src/header_tag.h                  \
		tag/id3lib/src/mp3_header.h                  \
		tag/id3lib/src/tag_impl.h                    \
		tag/id3lib/src/spec.h                        \
		tag/id3lib/include/id3.h					 \
		tag/id3lib/include/id3/field.h                       \
		tag/id3lib/include/id3/id3lib_frame.h                \
		tag/id3lib/include/id3/globals.h                     \
		tag/id3lib/include/id3/misc_support.h                \
		tag/id3lib/include/id3/reader.h                      \
		tag/id3lib/include/id3/readers.h                     \
		tag/id3lib/include/id3/sized_types.h                 \
		tag/id3lib/include/id3/tag.h                         \
		tag/id3lib/include/id3/writer.h                      \
		tag/id3lib/include/id3/writers.h                     \
		tag/id3lib/include/id3/utils.h                       \
		tag/id3lib/include/id3/id3lib_streams.h              \
		tag/id3lib/include/id3/id3lib_strings.h	             \
		tag/id3lib/include/id3/helpers.h                     \
		tag/id3lib/include/id3/io_decorators.h               \
		tag/id3lib/include/id3/io_helpers.h                  \
		tag/id3lib/include/id3/io_strings.h                  \
		sqlite_source/sqlite3.h

SOURCES		= \
		configuration.cpp \
		configurationdialog.cpp \
		main.cpp \
		debug.cpp \
		media.cpp \
		mediaidentifier.cpp \
		mediadatabase.cpp \
		mediainfodialog.cpp \
		mplayer.cpp \
		quasar.cpp \
		skinpanel.cpp \		
		skinnedbutton.cpp \
		skinnedlabel.cpp \
		skinnedtimelabel.cpp \
		skin.cpp \
		skinmanager.cpp \
		operationpanel.cpp \
		toolpanel.cpp \	
		coverartprovider.cpp \
		coverart.cpp \		
		playinfo.cpp \
		imagefx.cpp \
		videoplaybackwindow.cpp \
		playlist.cpp \
		playlistmanager.cpp \
		playlistoverview.cpp \
		playlisttoolpanel.cpp \
		seekbar.cpp \
		signalhandler.cpp \
		aboutdialog.cpp \
		keynames.cpp \
		action.cpp \
		textencoding.cpp \
		textviewdialog.cpp \
		compat/config.cpp \
		compat/resource.cpp \
		config/generalconfig.cpp \
		config/mplayerconfig.cpp \
		config/formatsconfig.cpp \
		config/hotkeyconfig.cpp \
		config/identifykeydialog.cpp \
		config/uiconfig.cpp \
		config/coverartconfig.cpp \
		tkmix/qtkfiledialog.cpp \
		system_volume.cpp \
		tag/id3lib/src/field.cpp                     \
		tag/id3lib/src/field_binary.cpp              \
		tag/id3lib/src/field_integer.cpp             \
		tag/id3lib/src/field_string_ascii.cpp        \
		tag/id3lib/src/field_string_unicode.cpp      \
		tag/id3lib/src/frame.cpp                     \
		tag/id3lib/src/frame_impl.cpp                \
		tag/id3lib/src/frame_parse.cpp               \
		tag/id3lib/src/frame_render.cpp              \
		tag/id3lib/src/globals.cpp                   \
		tag/id3lib/src/header.cpp                    \
		tag/id3lib/src/header_frame.cpp              \
		tag/id3lib/src/header_tag.cpp                \
		tag/id3lib/src/helpers.cpp                   \
		tag/id3lib/src/io.cpp                        \
		tag/id3lib/src/io_decorators.cpp             \
		tag/id3lib/src/io_helpers.cpp                \
		tag/id3lib/src/misc_support.cpp              \
		tag/id3lib/src/mp3_parse.cpp                 \
		tag/id3lib/src/readers.cpp                   \
		tag/id3lib/src/spec.cpp                      \
		tag/id3lib/src/tag.cpp                       \
		tag/id3lib/src/tag_file.cpp                  \
		tag/id3lib/src/tag_find.cpp                  \
		tag/id3lib/src/tag_impl.cpp                  \
		tag/id3lib/src/tag_parse.cpp                 \
		tag/id3lib/src/tag_parse_lyrics3.cpp         \
		tag/id3lib/src/tag_parse_musicmatch.cpp      \
		tag/id3lib/src/tag_parse_v1.cpp              \
		tag/id3lib/src/tag_render.cpp                \
		tag/id3lib/src/utils.cpp                     \
		tag/id3lib/src/writers.cpp					 \
		sqlite_source/sqlite3.c

TRANSLATIONS	= \
		quasar_ja.ts

MOC_DIR		= .moc
#OBJECTS_DIR	= objs

#MOC_DIR		= /hdd3/Playground/sources/zumbi/moc
#OBJECTS_DIR	= /hdd3/Playground/sources/zumbi/objs
